# frozen_string_literal: true

module Hermes
  class HousingRequestEntityForm
    include Hermes::Concerns::CommonFieldsConcern

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    field :active, type: Boolean, default: true
    field :entity_id, type: BSON::ObjectId
    field :housing_request_id, type: BSON::ObjectId
    field :applicant_id, type: BSON::ObjectId
    field :doc_id, type: BSON::ObjectId
    field :consent_fields, type: Array, default: []
    field :level, type: Integer
    field :delegation, type: Boolean, default: false
    field :delegation_signed, type: Boolean, default: true

    belongs_to :entity_form_level, class_name: 'Hermes::HousingRequestEntityFormLevel', foreign_key: :entity_form_level_id, index: true

    validates_presence_of :housing_request_id#, :entity_form_level_id
    validates_uniqueness_of :housing_request_id

    common_indexes *%i[
      active
      entity_id
      housing_request_id
      applicant_id
      doc_id
      consent_fields
      level
      delegation
      delegation_signed
    ]

    before_save :before_save_actions
    after_save :after_save_actions

    private

    def set_applicant_id
      request = Hermes::HousingRequest.find_by(id: self.housing_request_id)
      self.applicant_id = request.get_applicant_used_for_siret.id if request
    end

    def set_entity_id
      entity = Hermes::Entity.find_by(id: entity_form_level&.entity_id)
      self.set(entity_id: entity.id, updated_at: DateTime.current) if entity
    end

    def set_level_and_delegation
      self.set(
        level: entity_form_level&.level, 
        delegation: entity_form_level&.delegation, 
        updated_at: DateTime.current
      )
    end

    def update_housing_request_consent_level
      request = Hermes::HousingRequest.find_by(id: self.housing_request_id)
      request.update(consent_level: self.level) if request
    end

    def notify_delegation_for_applicant
      if delegation_changed? && delegation_change.last
        housing_request = Hermes::HousingRequest.find_by(id: housing_request_id)
        if housing_request && !delegation_signed
          housing_request.notify(key: NOTIFICATION_KEYS[:housing_request][:delegation_signed_required], channels: ['email'], main_only: true)
        end
      end
    end

    def before_save_actions
      set_applicant_id if housing_request_id_changed?
    end

    def after_save_actions
      if entity_form_level_id_changed?
        set_level_and_delegation
        set_entity_id
      end
      update_housing_request_consent_level
      # Temporary_disabled_devops_7390
      # notify_delegation_for_applicant
    end
  end
end